#!/bin/sh python3
"""	
This script plots the Fourier Transform from Lars data and compares to GTSM .

""" 
#%%
import os
import sys
# change path later based on location of Results folder
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from Analysis.Results import common_functions as cf
from Analysis.Results.Result_1 import reading_n_processing as rnp
from Analysis.Results.Result_1 import Buoy_Direct_TA as bdt
from Analysis.Results.Result_1 import Model_Buoy_TA as mbt
import xarray as xr
import importlib
importlib.reload(cf)
#%%
def get_TC_FFT(Uibt,Vibt,sampling_period,TCname,TC_value):
   
    (Ubt_amp,Ubt_ph,U_fvec,U_tvec)=cf.FFTsig(Uibt,sampling_period)
    (Vbt_amp,Vbt_ph,V_fvec,V_tvec)=cf.FFTsig(Vibt,sampling_period)
    const=2 #amp and phase
    Ub=np.zeros((len(TCname),const))
    Vb=np.zeros((len(TCname),const))
    #getting the index of the tidal constituents in the time vector of the FFT
    for i in range(len(TCname)):
        index=np.argmin(np.abs(U_tvec-TC_value[i]))
        Ub[i,0]=Ubt_amp[index]
        Ub[i,1]=Ubt_ph[index]
        Vb[i,0]=Vbt_amp[index]
        Vb[i,1]=Vbt_ph[index]
    #onverting the phase from -180 to 180 to 0 to 360
    Ub[:,1]=np.mod(Ub[:,1],360)
    Vb[:,1]=np.mod(Vb[:,1],360)
    return(Ub,Vb,U_tvec)
#%%
#reading the buoy data from Lars
folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
Bnum='16'
dt=15 #mins
fig_path='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Figures/'
sampling_period=dt/60 #hours
TC=cf.Tidal_const()
TC_value=np.fromiter(TC.values(),dtype=float)
TCname=TCname=np.array(list(TC.keys()))
(Xib,Yib,Uibt,Vibt,Tib)=rnp.get_buoy_tidal_data(folder,Bnum)
#doing a fourier transform on the data and obtaining the amplitude and phase of the tidal constituents
(Ubt_amp,Ubt_ph,U_fvec,U_tvec)=cf.FFTsig(Uibt,sampling_period)
(Vbt_amp,Vbt_ph,V_fvec,V_tvec)=cf.FFTsig(Vibt,sampling_period)
#%%
#plotting the FFT of the data
# fig=plt.figure(figsize=(10,10))
# ax1=fig.add_subplot(2,1,1)
# ax1.plot(U_tvec,Ubt_amp,label='U')
# ax1.plot(V_tvec,Vbt_amp,label='V')
# ax1.vlines(TC_value,0,np.max([Ubt_amp,Vbt_amp]),colors='k',linestyles='dashed',label='Tidal constituents')
# ax1.set_xlim([10,30])
# ax1.legend()
# ax1.set_xlabel('Frequency in hours')
# ax1.set_ylabel('Amplitude in m/s')
# ax2=fig.add_subplot(2,1,2)
# #convert phase to degrees 0 to 360
# Ubt_ph=np.mod(Ubt_ph,360)
# Ubt_ph[Ubt_ph<0]=Ubt_ph[Ubt_ph<0]+360
# Vbt_ph=np.mod(Vbt_ph,360)
# Vbt_ph[Vbt_ph<0]=Vbt_ph[Vbt_ph<0]+360
# ax2.plot(U_tvec,Ubt_ph,label='U')
# ax2.plot(V_tvec,Vbt_ph,label='V')
# ax2.vlines(TC_value,0,360,colors='k',linestyles='dashed',label='Tidal constituents')
# ax2.set_xlim([10,30])
# ax2.legend()
# ax2.set_ylabel('Phase in degrees')
# fig.savefig(path1+'/Analysis/Results/Result_1/FFT_buoy'+Bnum+'.jpg')
# %%
#saving the data in a dataframe
TC_Ub,TC_Vb,Tvec=get_TC_FFT(Uibt,Vibt,sampling_period,TCname,TC_value)
#%%
#reading model-buoydata and getting the amplitude and phase of the tidal constituennts at the buoy location
#obtaining the velocities
def get_model_buoy(Bnum,TCname,TC_value):
    model_param_file='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Model_Buoy_Method_Results/BUOY_'+Bnum+'_opt.csv'
    mod_data=pd.read_csv(model_param_file)
    opt_param_u=mod_data['Value_GD'].values[0:2*len(TCname)]
    opt_param_v=mod_data['Value_GD'].values[2*len(TCname):4*len(TCname)]
    comp='U'
    Mfile='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Model_Buoy_Method_Results/BUOY_'+Bnum+'_M_'+comp+'.nc'
    Mdata=xr.open_dataset(Mfile)
    M=Mdata['M'].values
    #U initial guess
    init_param=np.ones(2*len(TCname))
    init_param[1::2]=0.0
    Umod_init=mbt.get_velocities(M,init_param)
    #U_opt 
    Umod_opt=mbt.get_velocities(M,opt_param_u)
    #V initial guess
    init_param=np.ones(2*len(TCname))
    init_param[1::2]=0.0
    comp='V'
    Mfile='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Model_Buoy_Method_Results/BUOY_'+Bnum+'_M_'+comp+'.nc'
    Mdata=xr.open_dataset(Mfile)
    M=Mdata['M'].values
    Vmod_init=mbt.get_velocities(M,init_param)
    #V_opt 
    Vmod_opt=mbt.get_velocities(M,opt_param_v)
    #matrox of TC
    TC_U_opt,TC_V_opt,Tvec=get_TC_FFT(Umod_opt.ravel(),Vmod_opt.ravel(),sampling_period,TCname,TC_value)
    TC_U_ini,TC_V_ini,Tvec=get_TC_FFT(Umod_init.ravel(),Vmod_init.ravel(),sampling_period,TCname,TC_value)
    return(TC_U_opt,TC_V_opt,TC_U_ini,TC_V_ini)

TC_U_opt,TC_V_opt,TC_U_ini,TC_V_ini=get_model_buoy(Bnum,TCname,TC_value)
## computing the 
# %%

#plotting the amplitude and phase of the tidal constituents 
def plot_avg_amp_phase(TC_Ub,TC_Vb,TC_U_mod,TC_V_mod,TCname,fig_name):

    fig=plt.figure(figsize=(10,10))
    ax1=fig.add_subplot(1,2,1)
    ax1.set_title('Amplitude (m)')
    ax1.xaxis.set_label_text('Observed-Buoy')
    ax1.yaxis.set_label_text('Model-Buoy')

    #plotting U amp for all TC
    U_color='b'
    V_color='g'
    #market list based on TC
    marker_list=['o','^','+','*']
    for i in range(len(TCname)):
        TCi=TCname[i]
        ax1.scatter(TC_Ub[i,0],TC_U_mod[i,0],label=TCname[i],marker=marker_list[i],color=U_color)
        ax1.scatter(TC_Vb[i,0],TC_V_mod[i,0],label=TCname[i],marker=marker_list[i],color=V_color)
        #draw a straight line y=x
    lims = [
        np.min([0, 0]),  # min of both axes
        np.max([0.3,0.3]),  # max of both axes
    ]
    ax1.plot(lims, lims, 'k-', alpha=0.5, zorder=0)
    ax1.set_aspect('equal')
    ax1.set_xlim(lims)
    ax1.set_ylim(lims)
    #custom legend for maarkers and colors separately
    U_patch = mpatches.Patch(color=U_color, label='U')
    V_patch = mpatches.Patch(color=V_color, label='V')
    legend1=plt.legend(handles=[U_patch,V_patch])
    ax1.add_artist(legend1)
    handles = [plt.Line2D([], [], marker=marker, linestyle='None', label=label,color='black') for marker, label in zip(marker_list, TCname)]
    ax1.legend(handles=handles,loc='lower right')
    #plotting the phase
    ax2=fig.add_subplot(1,2,2)
    ax2.set_title('Phase(degrees)')
    ax2.xaxis.set_label_text('Observed-Buoy')
    ax2.yaxis.set_label_text('Model-Buoy')
    for i in range(len(TCname)):
        TCi=TCname[i]
        ax2.scatter(TC_Ub[i,1],TC_U_mod[i,1],label=TCname[i],marker=marker_list[i],color=U_color)
        ax2.scatter(TC_Vb[i,1],TC_V_mod[i,1],label=TCname[i],marker=marker_list[i],color=V_color)
    lims=[np.min([0, 0]),np.max([380, 380])]
    ax2.plot(lims, lims, 'k-', alpha=0.75, zorder=0)
    ax2.set_aspect('equal')
    ax2.set_xlim(lims)
    ax2.set_ylim(lims)
    #custom legend for maarkers and colors separately
    U_patch = mpatches.Patch(color=U_color, label='U')
    V_patch = mpatches.Patch(color=V_color, label='V')
    legend1=plt.legend(handles=[U_patch,V_patch])
    ax2.add_artist(legend1)
    handles = [plt.Line2D([], [], marker=marker, linestyle='None', label=label,color='black') for marker, label in zip(marker_list, TCname)]
    ax2.legend(handles=handles,loc='lower right')
    fig.tight_layout()
    # fig.savefig(fig_name+'.jpg',dpi=300)

# %%
TC_U_mod=TC_U_opt
TC_V_mod=TC_V_opt
fig_name='Model_Buoy_'+Bnum+'_amp_phase_Optimized'
fig_name=fig_path+fig_name
plot_avg_amp_phase(TC_Ub,TC_Vb,TC_U_mod,TC_V_mod,TCname,fig_name)
#%%
TC_U_mod=TC_U_ini
TC_V_mod=TC_V_ini
fig_name='Model_Buoy_'+Bnum+'_amp_phase_Initial'
fig_name=fig_path+fig_name
plot_avg_amp_phase(TC_Ub,TC_Vb,TC_U_mod,TC_V_mod,TCname,fig_name)


# %%
#Comparison with the fitting method.
fitting_param_file='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Fitting_Method_Results/BUOY_'+Bnum+'_opt.csv'
fitting_data=pd.read_csv(fitting_param_file)
nparams=12
fitting_param_u=fitting_data['U_data'].values[:nparams*2]
fitting_param_v=fitting_data['V_data'].values[:nparams*2]
fitting_vel_file='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Fitting_Method_Results/BUOY_'+Bnum+'_velocities.csv'
fitting_vel_data=pd.read_csv(fitting_vel_file)
U_fitting=fitting_vel_data['U'].values
V_fitting=fitting_vel_data['V'].values
#%%
TC_U_fit,TC_V_fit,Tvec=get_TC_FFT(U_fitting,V_fitting,sampling_period,TCname,TC_value)
#%%
# %%
TC_U_mod=TC_U_fit
TC_V_mod=TC_V_fit
fig_name='Fitting_Buoy_'+Bnum+'_amp_phase'
fig_name=fig_path+fig_name
plot_avg_amp_phase(TC_Ub,TC_Vb,TC_U_mod,TC_V_mod,TCname,fig_name)


# %%
#plotting the amplitude and phase for eacach TC in a bar plot for observed, model and fitting and initial 
#here we have a group of 4 bars for each TC
# in each of the 4 bars, one is observed, one is model, one is fitting and one is initial_model
#and two subplots for amplitude and phase
fig=plt.figure(figsize=(14,6))
ax1=fig.add_subplot(1,2,1)
ax1.set_ylabel('Amplitude(m/s)')
ax1.set_xlabel('Tidal Constituents')
ax1.set_xticks(np.arange(0,4*len(TCname),4)+1)
ax1.set_xticklabels(TCname,rotation=90)
#setting the bar properties
bar_properties = {'align':'center','alpha':0.8,'edgecolor':'black'}
ax1.bar(np.arange(0,4*len(TCname),4),TC_Ub[:,0],width=0.5,label='Observed',**bar_properties)
ax1.bar(np.arange(0,4*len(TCname),4)+0.5,TC_U_opt[:,0],width=0.5,label='Model-Derived Fitting',**bar_properties)
ax1.bar(np.arange(0,4*len(TCname),4)+1,TC_U_fit[:,0],width=0.5,label='Direct Fitting',**bar_properties)
ax1.bar(np.arange(0,4*len(TCname),4)+1.5,TC_U_ini[:,0],width=0.5,label='Model (Initial)',**bar_properties)
ax1.legend()
ax2=fig.add_subplot(1,2,2)
ax2.set_ylabel('Phase(degrees)')
ax2.set_xlabel('Tidal Constituents')
ax2.set_xticks(np.arange(0,4*len(TCname),4)+1)
ax2.set_xticklabels(TCname,rotation=90)
ax2.bar(np.arange(0,4*len(TCname),4),TC_Ub[:,1],width=0.5,label='Observed',**bar_properties)
ax2.bar(np.arange(0,4*len(TCname),4)+0.5,TC_U_opt[:,1],width=0.5,label='Model-Derived Fitting',**bar_properties)
ax2.bar(np.arange(0,4*len(TCname),4)+1,TC_U_fit[:,1],width=0.5,label='Direct Fitting',**bar_properties)
ax2.bar(np.arange(0,4*len(TCname),4)+1.5,TC_U_ini[:,1],width=0.5,label='Model (Initial)',**bar_properties)
ax2.legend(loc='upper right')
fig.tight_layout()
fig.savefig(fig_path+'Compare_Bar'+Bnum+'.jpg',dpi=300)



# %%
