#!/bin/sh python3
''' 
computing the fouriert transform of the buoy data and plotting the amplitude and phase
Then compute the frequency shift and plot it
we use hatyan
'''
#%%
import os
import sys
# change path later based on location of Results folder
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import cartopy.feature as cfeature
import matplotlib.path as mpath
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from Analysis.Results import common_functions as cf
from Analysis.Results.Result_1 import reading_n_processing as rnp
from Data_Processing.Buoy_processing import reading_plotting as bscripts
import xarray as xr
import importlib
import hatyan

#%%

def get_buoy_velocities(buoy_df,sampling_period):
    #since buoy lon lat are given we first compute thier velocities
    dt=3*60*60 #sampling time step for each buoy observation.
    Ub=0;Vb=0
    # note that the initial velocities to be zero for data to be 121.
    Lon=buoy_df['Lon'].values
    Lat=buoy_df['Lat'].values
    for j in range(len(Lon)-1):
        dlon=Lon[j+1]-Lon[j]
        dlat=Lat[j+1]-Lat[j]
        lat=Lat[j];lon=Lon[j]
        # (dlatm,dlonm)=gf.latlon2meters(lat,lon,dlat,dlon)
        (dlatm,dlonm)=bscripts.latlon2meters(lat,dlat,dlon)
        Ub=np.append(Ub,(dlonm)/dt)
        # Ub=np.append(Ub,(0)/dt)
        Vb=np.append(Vb,(dlatm)/dt)
        # Vb=np.append(Vb,(0)/dt)
    #remove the mean
    Ub=Ub-np.mean(Ub)
    Vb=Vb-np.mean(Vb)
    # [Ubres,Ubt,Ubfil]=bscripts.LPfilter(numtaps,Ub)
    # [Vbres,Vbt,Vbfil]=bscripts.LPfilter(numtaps,Vb)
    # [Ubres,Ubt,Ubfil]=bscripts.HPfilter(sampling_period,Ub)
    # [Vbres,Vbt,Vbfil]=bscripts.HPfilter(sampling_period,Vb)
    [Ubres,Ubt,Ubfil]=bscripts.BPfilter(sampling_period,Ub)
    [Vbres,Vbt,Vbfil]=bscripts.BPfilter(sampling_period,Vb)  
    return(Ubres,Vbres)

#get baroclinic buoy data
def get_baroclinic_buoy_data():
    sampling_period=3
    # #testing for another buoy in the region
    # buoy_id_vec=['B2014_26_11']
    # #buoy data
    # buoy_location='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Data_Processing/Buoy_processing/IABP_processing/processed_data/'
    # #getting the buoy mid locations.
    # j=0
    # buoy_id=buoy_id_vec[j]
    # #buoy data
    # obs_data=pd.read_csv(buoy_location+buoy_id+'.csv',sep=',')
        
    # Ub=obs_data.Ub.values;Vb=obs_data.Vb.values
    # #bp filtereing the data
    # (Ubt,Ub,Ubo)=cf.BPfilter (sampling_period,Ub)
    # (Vbt,Vb,Vbo)=cf.BPfilter (sampling_period,Vb)
    ##New test
    buoy_file='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Baroclinic_Model_TOPAZ/model_buoy_data/300234064501660.L2_3.csv'
    obs_data=pd.read_csv(buoy_file,sep=',')
    Ubt,Vbt=get_buoy_velocities(obs_data,sampling_period)
    (Uobs_amp,Uobs_ph,U_fvec,U_tvec)=cf.FFTsig(Ubt,sampling_period)
    (V_obs_amp,Vobs_ph,V_fvec,V_tvec)=cf.FFTsig(Vbt,sampling_period)
    T_vec=pd.to_datetime(obs_data.Year*1000+obs_data.DOY.astype(int),format='%Y%j')
    ts_vec=pd.date_range(start=str(T_vec.values[0]),end=str(T_vec.values[-1]),freq='3H')[:]
    Ubtseries=pd.DataFrame(Ubt,index=ts_vec,columns=['values'])
    Vbtseries=pd.DataFrame(Vbt,index=ts_vec,columns=['values'])
    #compute Lon and lat by median
    Lonvec=obs_data.Lon.values;Latvec=obs_data.Lat.values
    buoy_lon=np.median(Lonvec);buoy_lat=np.median(Latvec)  
    return(buoy_lon,buoy_lat,Ubtseries,Vbtseries)

def get_barotropic_buoy_data():
    folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
    Bnum='16'
    dt=15 #mins
    sampling_period=dt/60 #hours
    (Xib,Yib,Uibt,Vibt,Tib)=rnp.get_buoy_tidal_data(folder,Bnum)
    #making dataframes
    Ubtseries=pd.DataFrame(Uibt,index=Tib,columns=['values'])
    Vbtseries=pd.DataFrame(Vibt,index=Tib,columns=['values'])
    #buoy location
    buoy_lon=np.median(Xib);buoy_lat=np.median(Yib)
    
    return(buoy_lon,buoy_lat,Ubtseries,Vbtseries)
    
def compute_FFT(useries,min_prominence=0.01):
    import scipy.signal as ssig
    from scipy.fft import fft, fftfreq
    y=useries.values.flatten()
    T=useries.index[1]-useries.index[0]
    N=len(y)
    # T=[3]
    T=[T.seconds/3600]
    yf = fft(y)
    ampvec = np.abs(yf)/N
    freq = fftfreq(N, T[0])
    # peaks, peaks_properties = ssig.find_peaks(ampvec[freq >=0], prominence=min_prominence)
    peaks, peaks_properties = ssig.find_peaks(ampvec[freq>=0],height=0.01)
    peak_freq =  freq[peaks]
    peak_amp = ampvec[peaks]
    return(peak_freq,peak_amp,ampvec,freq)



# %%
if __name__ == "__main__":
    #baroclinic data
    bclon,bclat,Ubct,Vbct=get_baroclinic_buoy_data()
    btlon,btlat,Ubtt,Vbtt=get_barotropic_buoy_data()
    

    #baroclinic FFT data
    #below is only for U velocity Need to plot fo V velocity as well in the same graph
    vel_type='U'
    if vel_type=='U':
        Ubct=Ubct
        peak_freq,peak_amp,ampvec,freq=compute_FFT(Ubct,min_prominence=0.01)
    else:
        Ubct=Vbct
        peak_freq,peak_amp,ampvec,freq=compute_FFT(Vbct,min_prominence=0.01)
    #plotting only peak_freqs around 12.42
    M2=12.42
    N=len(ampvec)
    peak_freq=1/peak_freq
    peak_select_freq=peak_freq[(peak_freq>=M2-1.5) & (peak_freq<=M2+2)]
    #select amp based on peak_select_freq
    peak_amp_select=peak_amp[(peak_freq>=M2-1.5) & (peak_freq<=M2+2)]

    #making a nice plot. first is to label the vertical line with M2 freq name
    # second is to write values of the peak freqs at the dots.
    # plotting the amp 
    #for V:
    fig,ax=plt.subplots(1,1,figsize=(12,5))
    ax.plot(peak_select_freq,100*peak_amp_select,'o',color='r',label='FFT peaks near M2')
    ax.text(peak_select_freq[0]+0.3,100*peak_amp_select[0],"%.2f" % peak_select_freq[0],fontsize=16,fontweight='bold')
    ax.text(peak_select_freq[1]-1.5,100*peak_amp_select[1],"%.2f" % peak_select_freq[1],fontsize=16,fontweight='bold')
    # ax.text(peak_select_freq[2]-1.3,100*peak_amp_select[2]+0.1,"%.2f" % peak_select_freq[2],fontsize=16,fontweight='bold')
    ax.plot(1/freq[:N//2], 100*ampvec[:N//2],label='FFT')
    plt.vlines(M2,0,9,linestyles='dashed',color='g')
    ax.text(M2+0.1,8.5,'M2',rotation=90,fontsize=16,fontweight='bold')
    plt.xlabel('Period (hours)',fontsize=16,fontweight='bold')
    plt.ylabel('Amplitude (cm/s)',fontsize=16,fontweight='bold')
    plt.xlim(10,30)
    
    plt.xticks(np.linspace(10,30,11),fontsize=16,fontweight='bold')
    plt.yticks(fontsize=16,fontweight='bold')
    plt.legend(fontsize=16,loc='upper right')
    #legend font weight bold
    plt.setp(ax.get_legend().get_texts(), fontsize='16',fontweight='bold')
    ax.grid()
    plt.title('Amplitude spectrum of '+vel_type+' velocity component',fontsize=16,fontweight='bold')
    
    fig.tight_layout()
    
    fig.savefig('Figures/FFT_baroclinic_buoy_'+vel_type+'_2018_3peaks.jpg',dpi=300)
    ## for U
    # fig,ax=plt.subplots(1,1,figsize=(12,5))
    # ax.plot(peak_select_freq,100*peak_amp_select,'o',color='r',label='FFT peaks near M2')
    # ax.text(peak_select_freq[0]+0.3,100*peak_amp_select[0],"%.2f" % peak_select_freq[0],fontsize=16,fontweight='bold')
    # ax.text(peak_select_freq[1]-1.5,100*peak_amp_select[1],"%.2f" % peak_select_freq[1],fontsize=16,fontweight='bold')
    # ax.text(peak_select_freq[2]-1.3,100*peak_amp_select[2]+0.1,"%.2f" % peak_select_freq[2],fontsize=16,fontweight='bold')
    # ax.plot(1/freq[:N//2], 100*ampvec[:N//2],label='FFT')
    # plt.vlines(M2,0,4,linestyles='dashed',color='g')
    # ax.text(M2+0.1,3.5,'M2',rotation=90,fontsize=16,fontweight='bold')
    # plt.xlabel('Frequency (hours)',fontsize=16,fontweight='bold')
    # plt.ylabel('Amplitude (cm/s)',fontsize=16,fontweight='bold')
    # plt.xlim(10,30)
    
    # plt.xticks(np.linspace(10,30,11),fontsize=16,fontweight='bold')
    # plt.yticks(fontsize=16,fontweight='bold')
    # plt.legend(fontsize=16,loc='upper right')
    # #legend font weight bold
    # plt.setp(ax.get_legend().get_texts(), fontsize='16',fontweight='bold')
    # ax.grid()
    # plt.title('FFT of Amplitude for '+vel_type+' velocity component',fontsize=16,fontweight='bold')
    
    # fig.tight_layout()
    
    # fig.savefig('Figures/FFT_baroclinic_buoy_'+vel_type+'_2018_3peaks.jpg',dpi=300)
    #computing the average of the velocity from the buoy.
    #%%
    #computing the shift
    def compute_metric_1(Ub):
        return(np.sqrt(np.mean(Ub.values**2)))
    def compute_metric_2(Ub):
        # return(np.mean(np.abs(Ub.values)))
        return(np.mean(Ub.values))
    def compute_metric_3(Ub,Vb):
        #magnitude of the mean or abs mean in each direction
        Ubabs_mean=np.mean(np.abs(Ub.values))
        Vbabs_mean=np.mean(np.abs(Vb.values))
        # return(np.sqrt(Ubabs_mean**2+Vbabs_mean**2))
        return(np.mean(np.sqrt(Ub.values**2+Vb.values**2)))
        # return(np.sqrt(Ub.values.mean()**2+Vb.values.mean()**2))


    def compute_freq_shift(U_avg,M2_lambda):
        #frequency shift.
        # M2_lambda=100*1000 #m
        M2_time_sec=12.42*3600 #sec
        M2_freq_sec=1/M2_time_sec
        omega_M2=2*np.pi*M2_freq_sec
        c=M2_lambda/M2_time_sec
        multiplier1=(c+U_avg)/(c)
        multiplier2=(c-U_avg)/(c)
        new_freq1=multiplier1*M2_freq_sec
        new_freq2=multiplier2*M2_freq_sec
        new_time_period1=1/new_freq1
        new_time_period2=1/new_freq2
        #average shift. is u_avg/\lambda
        #in freq so m2_lambda/m2_time /3600 to get in hours divide by 2pi
        avg_ang_fre_hshift=2*np.pi*U_avg/M2_lambda
        new_freq_1=omega_M2+avg_ang_fre_hshift
        new_freq_2=omega_M2-avg_ang_fre_hshift
        new_time_period1=2*np.pi/new_freq_1
        new_time_period2=2*np.pi/new_freq_2
        print('average shift is %.2f hours' % (1/avg_ang_fre_hshift/3600))
        print(new_time_period1/3600)
        print(new_time_period2/3600)

        print('new time period is %.2f hours' % (new_time_period1/3600))
        print('new time period is %.2f hours' % (new_time_period2/3600))
        return(new_time_period1/3600,new_time_period2/3600)
    
    #computing the shift for new velocity with different metric3 and different M2_lambda
    M2_lambda_vec=[50*1000,100*1000,150*1000,300*1000,1000*1000]
    # vel_type_vec=['U','V']
    # metric_vec=[compute_metric_1,compute_metric_2]
    # #making a dictionary to store the results and then create a dataframe
    # #different dictionaries for different metric. with index of lambda and columns of time period 1 and 2
    # # for u and v velocity
    
    # for metric in metric_vec:
    #     ResultsU={'Period1':[],'Period2':[],'M2_lambda':[],'Ub_avg':[]}
    #     ResultsV={'Period1':[],'Period2':[],'M2_lambda':[],'Vb_avg':[]}
    
    #     for vel_type in vel_type_vec:
    #         if vel_type=='U':
    #             Ub=Ubct
    #         else:
    #             Ub=Vbct
    #         Ub_avg=metric(Ub)
            
    #         for M2_lambda in M2_lambda_vec:
    #             new_time_period1,new_time_period2=compute_freq_shift(Ub_avg,M2_lambda)
    #             if vel_type=='U':
    #                 ResultsU['Period1'].append(new_time_period1)
    #                 ResultsU['Period2'].append(new_time_period2)
    #                 ResultsU['M2_lambda'].append(M2_lambda)
    #                 ResultsU['Ub_avg'].append(Ub_avg)

    #             else:
    #                 ResultsV['Period1'].append(new_time_period1)
    #                 ResultsV['Period2'].append(new_time_period2)
    #                 ResultsV['M2_lambda'].append(M2_lambda)
    #                 ResultsV['Vb_avg'].append(Ub_avg)
    #     if metric==compute_metric_1:
    #         ResultsU1=pd.DataFrame(ResultsU,index=['Lambda1','Lambda2','Lambda3','Lambda4'])
    #         ResultsV1=pd.DataFrame(ResultsV,index=['Lambda1','Lambda2','Lambda3','Lambda4'])
    #     elif metric==compute_metric_2:
    #         ResultsU2=pd.DataFrame(ResultsU,index=['Lambda1','Lambda2','Lambda3','Lambda4'])
    #         ResultsV2=pd.DataFrame(ResultsV,index=['Lambda1','Lambda2','Lambda3','Lambda4'])
    ##average shift
    ResultsUnet={'Period1':[],'Period2':[],'M2_lambda':[],'Ub_avg':[]}
    
    Ub_avg=compute_metric_3(Ubct,Vbct)
        
    for M2_lambda in M2_lambda_vec:
        new_time_period1,new_time_period2=compute_freq_shift(Ub_avg,M2_lambda)
        ResultsUnet['Period1'].append(new_time_period1)
        ResultsUnet['Period2'].append(new_time_period2)
        ResultsUnet['M2_lambda'].append(M2_lambda)
        ResultsUnet['Ub_avg'].append(Ub_avg)    
    
    ResultsUnet=pd.DataFrame(ResultsUnet,index=['Lambda1','Lambda2','Lambda3','Lambda4','Lambda5'])	
        ###############################
    #%%
    #barotropic FFT data
    # vel_type='U'
    #below is only for u velocity. Let us plot for v velocity as well.
    if vel_type=='U':
        peak_freq,peak_amp,ampvec,freq=compute_FFT(Ubtt,min_prominence=0.01)
    else:
        peak_freq,peak_amp,ampvec,freq=compute_FFT(Vbtt,min_prominence=0.01)
    N=len(ampvec)
    peak_freq=1/peak_freq
    peak_select_freq=peak_freq[(peak_freq>=M2-0.7) & (peak_freq<=M2+0.7)]
    peak_amp_select=peak_amp[(peak_freq>=M2-0.7) & (peak_freq<=M2+0.7)]
    fig,ax=plt.subplots(1,1,figsize=(12,5))
    ax.plot(peak_select_freq,100*peak_amp_select,'o',color='r',label='FFT peaks near M2')
    ax.text(peak_select_freq[0]+0.2,100*peak_amp_select[0]-0.2,"%.2f" % peak_select_freq[0],fontsize=16,fontweight='bold')
    ax.plot(1/freq[:N//2], 100*ampvec[:N//2],label='FFT')
    plt.vlines(M2,0,35,linestyles='dashed',color='g')
    ax.text(M2+0.1,32,'M2',rotation=90,fontsize=16,fontweight='bold')
    plt.xlabel('Period (hours)',fontsize=16,fontweight='bold')   
    plt.ylabel('Amplitude (cm/s)',fontsize=16,fontweight='bold')
    plt.xlim(10,30)
    plt.xticks(np.linspace(10,30,11),fontsize=16,fontweight='bold')
    plt.yticks(fontsize=16,fontweight='bold')
    plt.legend(fontsize=16,loc='upper right')
    #legend font weight bold
    plt.setp(ax.get_legend().get_texts(), fontsize='16',fontweight='bold')
    ax.grid()
    plt.title('Amplitude spectrum of '+vel_type+' velocity component',fontsize=16,fontweight='bold')
    fig.tight_layout()
    fig.savefig('Figures/FFT_barotropic_buoy'+vel_type+'.jpg',dpi=300)
    # %%
    #plotting the buoy lon and lat on a north polar sterographic projection
    projection=ccrs.NorthPolarStereo(central_longitude=0.0)

    fig=plt.figure(figsize=(8,4))
    ax = plt.axes(projection=projection)
    extent=[-90,90,50.0,90.0]
    ax.set_extent(extent,ccrs.PlateCarree())
    # # Compute a circle in axes coordinates, which we can use as a boundary
    # # for the map. We can pan/zoom as much as we like - the boundary will be
    # # permanently circular
    # theta = np.linspace(0, np.pi, 100)
    # center, radius = [0.5, 1.0], 0.5
    # verts = np.vstack([np.sin(theta), np.cos(theta)]).T
    # circle = mpath.Path(verts * radius + center)
    # # get cyclic coordinates for projection
    # ax.set_boundary(circle, transform = ax.transAxes)


    #setting the coastlines and land with resolution 50m
    # ax.add_feature(cfeature.LAND,facecolor='0.8',edgecolor='k',linewidth=0.5)
    ax.add_feature(cfeature.LAND,facecolor='k',edgecolor='k',linewidth=0.5)
    ax.add_feature(cfeature.COASTLINE,facecolor='none',edgecolor='k',linewidth=0.5)


    #setting the ticks
    # Define x ticks in Plate Carree coordinate system
    #defining same number of ticks in x and y direction
    xticks_pc = np.arange(extent[0], extent[1], 30)
    yticks_pc = np.arange(extent[2], extent[3], 10)
    gl = ax.gridlines(draw_labels=True, xlocs=xticks_pc, ylocs=yticks_pc,alpha=0.5)
        
    #adding the x and y labels
    lon_formatter = LongitudeFormatter(number_format='.1f',
                                       degree_symbol='',
                                       dateline_direction_label=True)
    lat_formatter = LatitudeFormatter(number_format='.1f',
                                      degree_symbol='')
    ax.xaxis.set_major_formatter(lon_formatter)
    ax.yaxis.set_major_formatter(lat_formatter)

    gl.right_labels = False
    gl.xlabel_style = {'size': 9, 'color': 'black'}
    gl.ylabel_style = {'size': 9, 'color': 'black'}
    gl.left_labels = True
    # ##Plotting the buoy trajectories
    scatter_opts = dict(s=100, marker='o', transform=ccrs.PlateCarree())
    ax.scatter(bclon,bclat,color='r',label='Baroclinic',**scatter_opts)
    ax.scatter(btlon,btlat,color='b',label='Barotropic',**scatter_opts)
    legend_properties = {'weight':'bold'}
    plt.legend(prop=legend_properties,fontsize=12)
    fig.tight_layout()
    # plt.savefig('Figures/buoy_locations.png',dpi=300,bbox_inches='tight')
    
    
    

# %%
