#!/usr/bin/env python
# -*- coding: utf-8 -*-
#%%
"""
This function is the base script where all the buoy time series is obtained  by the equation given in the paper.
In this case, we just compute the velocity of the buoy from the amplitude and phase estimates. The equations are:
U= A_o +dA/dX*(x-xo)+dA/dY*(y-yo)(cos(wt-phi+dphi/dX*(x-xo)+dphi/dY*(y-yo))

"""
#importing the required packages
import os
import sys
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from Analysis.Results import common_functions as cf
from Analysis.Results.Result_1 import reading_n_processing as rnp
import hatyan
from joblib import Parallel, delayed
from scipy.optimize import minimize

#%%


def compute_tidal_prediction(TC,A_net,phi_net,ts_vec):
    U_tc=np.zeros(len(ts_vec))   
    for i in range(len(ts_vec[:])):
        #evaluating the components
        const=pd.DataFrame({'A':A_net[:,i],'phi_deg':phi_net[:,i]},index=TC)
        # @
        try:
            if i==0 or i==1 or i==2:
                U_tci=hatyan.prediction(comp=const,times_ext=ts_vec[i:i+2],timestep_min=15)
                U_tc[i]=U_tci.values[0]
            else:
                U_tci=hatyan.prediction(comp=const,times_ext=ts_vec[i-3:i],timestep_min=15)
                U_tc[i]=U_tci.values[-1]

        except:
                Exception('provided component set contains nan values, prediction not possible')
                U_tc[i]=0.0
    return(U_tc)    
 

def computing_buoy_estimated_velocity(Xib,Yib,Tib,Amp_params,phi_params,TCname):
    #getting the time vector

    #computing X and Y difference vectors with forward differnce in x1-x0 and y1-y0
    dX=np.zeros(len(Xib))
    dX=Xib-Xib[0]
    dY=np.zeros(len(Yib))
    dY=Yib-Yib[0]

    # getting amplitude and phase parameters
    #Amp_params is of shape 4x3 where 4 is TC and 3 is A_0, dA/dX, dA/dY
    #Net amplitude is A_0+dA/dX*dX+dA/dY*dY
    #below Anet is then a 4xlen(T_vec) matrix
    A_net=np.zeros((len(Amp_params),len(Xib)))
    phi_net=np.zeros((len(phi_params),len(Xib)))
    for i in range(len(Xib)):
        A_net[:,i]=Amp_params[:,0]+Amp_params[:,1]*dX[i]+Amp_params[:,2]*dY[i]
        phi_net[:,i]=phi_params[:,0]+phi_params[:,1]*dX[i]+phi_params[:,2]*dY[i]
    # print(A_net)
    # print(phi_net.shape)
    #converting phi to degrees
    phi_net=phi_net*180/np.pi
    #computing the velocity vector by tidal prediction
    U_ts=compute_tidal_prediction(TCname,A_net,phi_net,Tib)
    return(U_ts)

def testing_buoy_estimated_velocity(Xib,Yib,Tib,TCname):
    Amp_params=np.zeros((len(TC),3))
    Amp_params[:,0]=np.array([0.03,0.01,0.002,0.002])
    Amp_params[:,1]=np.array([0.0,0.0,0.0,0.0])
    Amp_params[:,2]=np.array([0.0,0.0,0.0,0.0])
    phi_params=np.pi*np.ones((len(TC),3))
    U_ts=computing_buoy_estimated_velocity(Xib,Yib,Tib,Amp_params,phi_params,TCname)
    return(U_ts)

def compute_residuals(params,Xib,Yib,Tib,Uobs,TCname):
    Amp_params=params[0:4*3].reshape(4,3)
    phi_params=params[4*3:].reshape(4,3)
    U_ts=computing_buoy_estimated_velocity(Xib,Yib,Tib,Amp_params,phi_params,TCname)
    #residual
    resvec=Uobs-U_ts
    res=np.sum(resvec**2)
    return(res)

def compute_amp_phase_median(Xib,Yib,params_u):
    Amp_params=params_u[0:4*3].reshape(4,3)
    phi_params=params_u[4*3:].reshape(4,3)
    Xbm,Ybm=rnp.get_median_buoy(Xib,Yib)
    dX=Xbm-Xib[0]
    dY=Ybm-Yib[0]
    #getting the median amplitude and phase
    A_med=np.zeros(4)
    phi_med=np.zeros(4)
    for i in range(4):
        A_med[i]=Amp_params[i,0]+Amp_params[i,1]*dX+Amp_params[i,2]*dY
        phi_med[i]=phi_params[i,0]+phi_params[i,1]*dX+phi_params[i,2]*dY
    return(A_med,phi_med)

#%%

if __name__ == '__main__':
    
     #change based on location of the data
    folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
    Bnum='03'
    dt=15 #mins
    sampling_period=dt/60 #hours
    
    (Xib,Yib,Uibt,Vibt,Tib)=rnp.get_buoy_tidal_data(folder,Bnum)
    TC=cf.Tidal_const()
    TCname=np.fromiter(TC.keys(),dtype='U3')
    TCval=np.fromiter(TC.values(),dtype='float')
    #%%
    #testing the velocity computation
    U_ts=testing_buoy_estimated_velocity(Xib,Yib,Tib,TCname)


    #residuals
    




# %%
