#!/bin/sh python
# -*- coding: utf-8 -*-
#%%
"""
#now the idea is to obtain A(x,y) and phi(x,y) from the model. But instead of 
performing the optimzation for greenland for u and v separate is better.
The idea is to compute a U and V separate optimization. 
res=Ubres-Umod
Umod=M*param
M=[Acos(w1t-phi) Asin(w1t-phi)...]      # 8xlen(Xib)
This is the same as the previous one but with the linearised residuals.
"""
#importing the required packages
import os
import sys
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import datetime
import hatyan
import xarray as xr
from Analysis.Results.Result_1 import reading_n_processing as rnp
from Analysis.Results import common_functions as cf
from Analysis.Results import interpolating_TC as itc

#%%

def get_velocities(M,param):
    
    Umod=M@param.reshape((len(param),1))
    return(Umod)

# constructing a column vector of the matrix.

def compute_vel_component_vec(A_tc,phi_tc,ts_vec,TCi):
    U_tc=np.zeros(len(ts_vec))
    
    for i in range(len(ts_vec[:])):
        #evaluating the components
        const=pd.DataFrame({'A':A_tc[i],'phi_deg':phi_tc[i]},index=[TCi])
        # @
        try:
            if i==0 or i==1 or i==2:
                U_tci=hatyan.prediction(comp=const,times_ext=ts_vec[i:i+2],timestep_min=15)
                U_tc[i]=U_tci.values[0]
            else:
                U_tci=hatyan.prediction(comp=const,times_ext=ts_vec[i-3:i],timestep_min=15)
                U_tc[i]=U_tci.values[-1]
        except:
            Exception('provided component set contains nan values, prediction not possible')
            U_tc[i]=0.0
    return(U_tc)    

# constructing the matrix M

def compute_M_matrix(Xib,Yib,Tib,model_file,comp):

    mod_thc_data=xr.open_dataset(model_file)
    Lon=Xib;Lat=Yib

    TCname=mod_thc_data['TC'].values

    (A_U,A_V,phi_deg_U,phi_deg_V)=eval_TC_ts(Lon,Lat,mod_thc_data)
    if comp=='U':
        A=A_U
        phi=phi_deg_U
    else:
        A=A_V
        phi=phi_deg_V
    
    M=np.zeros((len(Tib),2*len(TCname)))
    for i in range(len(TCname)):
        TCi=TCname[i]
        A_tc=A[:,i]
        phi_tc=phi[:,i]
        print(A_tc.shape)
        u_tc_cos=compute_vel_component_vec(A_tc,phi_tc,Tib,TCi)
        # 90 deg phase shift for sin component
        u_tc_sin=compute_vel_component_vec(A_tc,phi_tc+90,Tib,TCi)
        M[:,2*i]=u_tc_cos
        M[:,2*i+1]=u_tc_sin
    return(M)

#model interpolated data.

#  function to process and compute velocities.
def interpolate_grid(buoy_lon,buoy_lat,mod_thc_data,TC):
    mod_longrid=mod_thc_data.Longitude.values
    mod_latgrid=mod_thc_data.Latitude.values
    TCind=np.where(np.array(mod_thc_data.TC.values)==TC)[0][0]
    U_TCamp=mod_thc_data.UAmp[:,TCind].values
    U_TCph=mod_thc_data.UPh[:,TCind].values
    V_TCamp=mod_thc_data.VAmp[:,TCind].values
    V_TCph=mod_thc_data.VPh[:,TCind].values
    # reshaping the data to 2D
    dim=int(np.sqrt(len(mod_longrid)))
    mod_longrid=mod_longrid.reshape(dim,dim);mod_latgrid=mod_latgrid.reshape(dim,dim)
    U_TCamp=U_TCamp.reshape(dim,dim);U_TCph=U_TCph.reshape(dim,dim)
    V_TCamp=V_TCamp.reshape(dim,dim);V_TCph=V_TCph.reshape(dim,dim)

    U_interp_amp,U_interp_ph=itc.THCinterp2(mod_longrid,mod_latgrid,U_TCamp,U_TCph,buoy_lon,buoy_lat,isdeg=True)
    V_interp_amp,V_interp_ph=itc.THCinterp2(mod_longrid,mod_latgrid,V_TCamp,V_TCph,buoy_lon,buoy_lat,isdeg=True)
    #changing the nan amp and phase to 0
    U_interp_amp[np.isnan(U_interp_amp)]=0
    V_interp_amp[np.isnan(V_interp_amp)]=0
    U_interp_ph[np.isnan(U_interp_ph)]=0
    V_interp_ph[np.isnan(V_interp_ph)]=0
    return(U_interp_amp,U_interp_ph,V_interp_amp,V_interp_ph)
# test
# (U_interp_amp,U_interp_ph,V_interp_amp,V_interp_ph)=interpolate_grid(buoy_lon,buoy_lat,mod_thc_data,'M2')


def eval_TC_ts(buoy_lon,buoy_lat,mod_thc_data):
    A_U=np.zeros((len(buoy_lon),len(mod_thc_data.TC.values)));A_V=np.zeros((len(buoy_lon),len(mod_thc_data.TC.values)))
    phi_deg_U=np.zeros((len(buoy_lon),len(mod_thc_data.TC.values)));phi_deg_V=np.zeros((len(buoy_lon),len(mod_thc_data.TC.values)))
    for i in range(len(mod_thc_data.TC.values)):
        TC=mod_thc_data.TC.values[i]
        (U_interp_amp,U_interp_ph,V_interp_amp,V_interp_ph)=interpolate_grid(buoy_lon,buoy_lat,mod_thc_data,TC)
        A_U[:,i]=U_interp_amp
        A_V[:,i]=V_interp_amp
        phi_deg_U[:,i]=U_interp_ph
        phi_deg_V[:,i]=V_interp_ph
    return(A_U,A_V,phi_deg_U,phi_deg_V)


    

#%%
if __name__ == '__main__':
    
     #change based on location of the data
    folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
    Bnum='16'
    dt=15 #mins
    sampling_period=dt/60 #hours
    
    (Xib,Yib,Uibt,Vibt,Tib)=rnp.get_buoy_tidal_data(folder,Bnum)
    TC=cf.Tidal_const()
    TCname=np.fromiter(TC.keys(),dtype='U3')
    TCval=np.fromiter(TC.values(),dtype='float')
    #model file
    model_file='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Calibration/Lars_Data/GTSM_TA_grid_May.nc'
    mod_thc_data=xr.open_dataset(model_file)
    M=compute_M_matrix(Xib,Yib,Tib,model_file,'U')
    init_param=np.ones(2*len(TCname))
    init_param[1::2]=0.0
    Umod=get_velocities(M,init_param)
    #%%
    res=np.linalg.norm(Uibt-Umod)
    fig=plt.figure()
    # fig.title('Residuals: '+str(res))
    # plt.plot(Tib,Uibt,'r')
    # plt.plot(Tib,Umod,'b')
    plt.plot(Tib,np.abs(Uibt-Umod.ravel()),'k')
    # plt.savefig('residuals.png')

    
# %%
