#!/usr/bin/env python
# -*- coding: utf-8 -*-
#%%
"""
performing a least squares optimization for the buoys using the direct fitting method
"""
#importing the required packages
#importing the required packages
import os
import sys
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from Analysis.Results import common_functions as cf
from Analysis.Results.Result_1 import reading_n_processing as rnp
from Analysis.Results.Result_1 import Buoy_Direct_TA as bdt
import hatyan
from joblib import Parallel, delayed
from scipy.optimize import minimize  
from scipy.optimize import Bounds
import xarray as xr

#%%
def getting_amp_phase_vec(params,Xib,Yib):
    Lon=Xib
    Lat=Yib
    #computing X and Y difference vectors with forward differnce in x1-x0 and y1-y0
    dX=np.zeros(len(Lon))
    dX=Lon-Lon[0]
    dY=np.zeros(len(Lat))
    dY=Lat-Lat[0]

    #Amp_params is of shape 4x3 where 4 is TC and 3 is A_0, dA/dX, dA/dY
    #Net amplitude is A_0+dA/dX*dX+dA/dY*dY
    #below Anet is then a 4xlen(T_vec) matrix
    Amp_params=params[0:4*3].reshape(4,3)
    phi_params=params[4*3:].reshape(4,3)
    A_net=np.zeros((len(Amp_params),len(Lon)))
    phi_net=np.zeros((len(phi_params),len(Lon)))
    for i in range(len(Lon)):
        A_net[:,i]=Amp_params[:,0]+Amp_params[:,1]*dX[i]+Amp_params[:,2]*dY[i]
        phi_net[:,i]=phi_params[:,0]+phi_params[:,1]*dX[i]+phi_params[:,2]*dY[i]
    return(A_net,phi_net)

def constraint1(params,Xib,Yib,Uobs):
    A_net,phi_net=getting_amp_phase_vec(params,Xib,Yib)
    A_net=A_net.ravel()
    # print(A_net)
    constraint_value=A_net
    return(constraint_value)    
def constraint2(params,Xib,Yib,Uobs):
    A_net,phi_net=getting_amp_phase_vec(params,Xib,Yib)
    A_net=A_net.ravel()
    constraint_value=np.max(Uobs)-A_net
    return(constraint_value)   
def constraint3(params,Xib,Yib,Uobs):
    A_net,phi_net=getting_amp_phase_vec(params,Xib,Yib) 
    phi_net=phi_net.ravel()
    constraint_value=phi_net
    return(constraint_value)   
def constraint4(params,Xib,Yib,Uobs):
    A_net,phi_net=getting_amp_phase_vec(params,Xib,Yib)
    phi_net=phi_net.ravel()
    limit=2*np.pi
    constraint_value=limit-phi_net
    return(constraint_value)   



def perform_optimization(folder,Bnum,int_param,TCname):
    (Xib,Yib,Uibt,Vibt,Tib)=rnp.get_buoy_tidal_data(folder,Bnum)
    
    comp='U'
    Uobs=Uibt
    bound_min=-np.inf*np.ones((len(int_param)))
    bound_min[0::3]=0.0
    bound_max=np.inf*np.ones((len(int_param)))
    bound_max[0::3]=np.max(Uobs)
    bound_max[12::3]=2*np.pi
    # bounds=(bound_min,bound_max)  
    bounds=Bounds(bound_min,bound_max)
    constraint_list=[{'type':'ineq','fun':constraint1,'args':(Xib,Yib,Uobs)},{'type':'ineq','fun':constraint2,'args':(Xib,Yib,Uobs)},{'type':'ineq','fun':constraint3,'args':(Xib,Yib,Uobs)},{'type':'ineq','fun':constraint4,'args':(Xib,Yib,Uobs)}]
    result=minimize(bdt.compute_residuals,x0=int_param,args=((Xib,Yib,Tib,Uobs,TCname)),method='SLSQP',constraints=constraint_list,bounds=bounds)
    result_u=result
    comp='V'
    Uobs=Vibt
    bound_min=-np.inf*np.ones((len(int_param)))
    bound_min[0::3]=0.0
    bound_max=np.inf*np.ones((len(int_param)))
    bound_max[0::3]=np.max(Uobs)
    bound_max[12::3]=2*np.pi
    bounds=Bounds(bound_min,bound_max)
    result=minimize(bdt.compute_residuals,x0=int_param,args=((Xib,Yib,Tib,Uobs,TCname)),method='SLSQP',constraints=constraint_list,bounds=bounds)
    result_v=result
    params_u=result_u.x
    cost=result_u.success
    nfev=result_u.nfev
    u_data=np.hstack((params_u,cost,nfev))
    params_v=result_v.x
    cost=result_v.success
    nfev=result_v.nfev
    v_data=np.hstack((params_v,cost,nfev))
    data={'U_data':u_data,'V_data':v_data}

    return(data,Xib,Yib)



def save_data_nc_file(data,data_median,TCname):
    #nc file with coordinates TCname, params ['zero',grad_x,grad_y]
    U_data=data['U_data'];V_data=data['V_data']
    #vectors of amplitude and phase of size 4x3 (4 tidal constituents and 3 parameters)
    U_Amp_params=np.reshape(U_data[0:12],(4,3))
    V_Amp_params=np.reshape(V_data[0:12],(4,3))
    U_phi_params=np.reshape(U_data[12:24],(4,3))
    V_phi_params=np.reshape(V_data[12:24],(4,3))
    U_A_med=data_median['U_A_med'];U_phi_med=data_median['U_phi_med']
    V_A_med=data_median['V_A_med'];V_phi_med=data_median['V_phi_med']
    #xrarray dataset
    ds=xr.Dataset({'U_Amp_params':(['TCname','params'],U_Amp_params),'V_Amp_params':(['TCname','params'],V_Amp_params),'U_phi_params':(['TCname','params'],U_phi_params),'V_phi_params':(['TCname','params'],V_phi_params),'U_A_med':(['TCname'],U_A_med),'U_phi_med':(['TCname'],U_phi_med),'V_A_med':(['TCname'],V_A_med),'V_phi_med':(['TCname'],V_phi_med)},
                  coords={'TCname':TCname,'params':['zero','grad_x','grad_y']},
                  attrs={'description':'The 6 Amplitude and Phase parameters for U and V components of the tidal currents for 4 tidal constituents. Also, a amplitude and phase for a median location for U and V components for all the tidal constituents'})
    return(ds)
def perform_optimization_parallel(i,j):
    index=j*num_cores+i
    Bnum=buoy_id_vec[index]
    buoy_id= 'BUOY_'+str(Bnum)
    init_param = np.zeros((24))
    # init_param[0::3]=0.0
    TC=cf.Tidal_const()
    TCname=np.fromiter(TC.keys(),dtype='U3')
    TCval=np.fromiter(TC.values(),dtype='float')
    
    data,Xib,Yib = perform_optimization(folder,Bnum,init_param,TCname)
    #save csv as backup
    data_df=pd.DataFrame(data)
    data_df.to_csv(opt_location + buoy_id+'_opt.csv')
    # #median amplitude and phase
    params_u=data['U_data'][0:24]	
    params_v=data['V_data'][0:24]
    U_A_med,U_phi_med=bdt.compute_median_amp_phase(Xib,Yib,params_u)
    V_A_med,V_phi_med=bdt.compute_median_amp_phase(Xib,Yib,params_v)
    data_median={'U_A_med':U_A_med,'U_phi_med':U_phi_med,'V_A_med':V_A_med,'V_phi_med':V_phi_med}
    # #save the data in a netcdf file
    ds=save_data_nc_file(data,data_median,TCname)
    ds.to_netcdf(opt_location + buoy_id+'_opt.nc')
    #
    return buoy_id

#%%
if __name__ == '__main__':
    
    folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
    dt=15 #mins
    sampling_period=dt/60 #hours
   
    buoy_id_vec=['03','16']
    opt_location='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Fitting_Method_Results/'
    

    #%%

    # create a csv file to store the results
    data={}

    num_cores=2
    inputs=range(0,num_cores)
    nParts=int(len(buoy_id_vec)/num_cores)
    for j in range(nParts):
        buoy_id=Parallel(n_jobs=num_cores)(delayed(perform_optimization_parallel)(i,j) for i in inputs)
        