#!/usr/bin/env python
# -*- coding: utf-8 -*-
#%%
"""
performing the optimzation for greenland for u and v separate is better.
"""
#importing the required packages
import os
import sys
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import datetime
import hatyan
import xarray as xr
from Analysis.Results.Result_1 import reading_n_processing as rnp
from Analysis.Results import common_functions as cf
from Analysis.Results import interpolating_TC as itc
from Analysis.Results.Result_1 import Model_Buoy_TA as mbta
from joblib import Parallel, delayed
from scipy.optimize import minimize

#%%

def compute_alpha_beta(opt_param_hat):
    # getting alpha beta consts
    alpha=np.sqrt(opt_param_hat[0::2]**2+opt_param_hat[1::2]**2)
    beta=-np.arctan2(opt_param_hat[1::2],opt_param_hat[0::2])*180/np.pi
    param=np.zeros((len(alpha)*2))
    param[0::2]=alpha
    param[1::2]=beta
    return(param)

#constraints
def constraint1(params):
    alpha_beta_param=compute_alpha_beta(params)
    alpha=alpha_beta_param[0::2]
    beta=alpha_beta_param[1::2]
    limit=5
    constraint_value=limit-alpha[0]
    return(constraint_value)    
def constraint2(params):
    alpha_beta_param=compute_alpha_beta(params)
    alpha=alpha_beta_param[0::2]
    beta=alpha_beta_param[1::2]
    limit=5
    constraint_value=limit-alpha[1]
    return(constraint_value)   
def constraint3(params):
    alpha_beta_param=compute_alpha_beta(params)
    alpha=alpha_beta_param[0::2]
    beta=alpha_beta_param[1::2]
    limit=5
    constraint_value=limit-alpha[2]
    return(constraint_value)   
def constraint4(params):
    alpha_beta_param=compute_alpha_beta(params)
    alpha=alpha_beta_param[0::2]
    beta=alpha_beta_param[1::2]
    limit=5
    constraint_value=limit-alpha[3]
    return(constraint_value)  

def residual_sum_sq(param,M,Ubres,Vbres,comp):
    if comp=='U':
        Umod=M@(param.reshape((len(param),1)))
        
        res=Ubres-Umod.ravel()
        # print(Ures.shape)
    else:
        Vmod=M@(param.reshape((len(param),1)))
        res=Vbres-Vmod.ravel()
    res_sum=np.sum(res**2)
    return(res_sum)

def perform_optimization(folder,Bnum,int_param,model_file):

    Xib,Yib,Uibt,Vibt,Tib=rnp.get_buoy_tidal_data(folder,Bnum)
    constraint_list=[{'type':'ineq','fun':constraint1},{'type':'ineq','fun':constraint2},{'type':'ineq','fun':constraint3},{'type':'ineq','fun':constraint4}]    
    comp='U'
    M=mbta.compute_M_matrix(Xib,Yib,Tib,model_file,comp)

    optimization_result_u=minimize(residual_sum_sq,x0=int_param,args=((M,Uibt,Vibt,comp)),method='SLSQP',constraints=constraint_list)
    result_u=optimization_result_u.x
    resid_u=optimization_result_u.fun
    rank_u=optimization_result_u.success
    s_u=optimization_result_u.nfev
    comp='V'
    M=mbta.compute_M_matrix(Xib,Yib,Tib,model_file,comp)

    optimization_result_v=minimize(residual_sum_sq,x0=int_param,args=((M,Uibt,Vibt,comp)),method='SLSQP',constraints=constraint_list)
    result_v=optimization_result_v.x
    resid_v=optimization_result_v.fun
    rank_v=optimization_result_v.success
    s_v=optimization_result_v.nfev
    
    param_u=compute_alpha_beta(result_u)
    param_v=compute_alpha_beta(result_v)
    
    csv_index_hat=['Gamma_U_M2','Delta_U_M2','Gamma_U_S2','Delta_U_S2','Gamma_U_O1','Delta_U_O1','Gamma_U_K1','Delta_U_K1','Gamma_V_M2','Delta_V_M2','Gamma_V_S2','Delta_V_S2','Gamma_V_O1','Delta_V_O1','Gamma_V_K1','Delta_V_K1','Rank_U','res_vec_U','Rank_V','res_vec_V']
    csv_index=['Amp_U_M2','Ph_U_M2','Amp_U_S2','Ph_U_S2','Amp_U_O1','Ph_U_O1','Amp_U_K1','Ph_U_K1','Amp_V_M2','Ph_V_M2','Amp_V_S2','Ph_V_S2','Amp_V_O1','Ph_V_O1','Amp_V_K1','Ph_V_K1','Fun_eval_U','res_vec_U','Fun_eval_V','res_vec_V']
    res_fun_val=[rank_u,resid_u,rank_v,resid_v]
    param_hat=np.append(result_u,result_v)
    param_hat=np.append(param_hat,res_fun_val)
    param=np.append(param_u,param_v)
    param=np.append(param,res_fun_val)
    data={'Type_GD':csv_index_hat,'Value_GD':param_hat,'Type_AP':csv_index,'Value_AP':param}
    return(data)

def perform_optimization_parallel(i,j):  
    index=j*num_cores+i
    Bnum=buoy_id_vec[index]
    buoy_id= 'BUOY_'+str(Bnum)
    init_param = np.ones((8))
    init_param[1::2] = 0.0
    # init_param[0::3]=0.0
    TC=cf.Tidal_const()
    TCname=np.fromiter(TC.keys(),dtype='U3')
    TCval=np.fromiter(TC.values(),dtype='float')
    data = perform_optimization(folder,Bnum,init_param,model_file)
    data_df=pd.DataFrame(data)
    data_df.to_csv(opt_location + buoy_id+'_opt.csv')
    return buoy_id
#%%
if __name__ == '__main__':
    
    folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
    dt=15 #mins
    sampling_period=dt/60 #hours
   
    buoy_id_vec=['03','16']
    opt_location='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Results/Result_1/Model_Buoy_Method_Results/'
    model_file='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Analysis/Calibration/Lars_Data/GTSM_TA_grid_May.nc'
    

    #%%

    # create a csv file to store the results
    data={}

    num_cores=2
    inputs=range(0,num_cores)
    nParts=int(len(buoy_id_vec)/num_cores)
    buoy_id=perform_optimization_parallel(0,0)
    for j in range(nParts):
        buoy_id=Parallel(n_jobs=num_cores)(delayed(perform_optimization_parallel)(i,j) for i in inputs)
        
# %%
