#!/bin/sh python3
"""	
This script reads in the buoy data of Lars for the spitzbergen region and obtains the bandpass filtered buoy data.
It also does a fourier transform on the data and plots the amplitude and phase.

""" 
#%%
import os
import sys
# change path later based on location of Results folder
sys.path.append('/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration')
path1=sys.path[-1]
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from Analysis.Results import common_functions as cf
import datetime
import importlib
importlib.reload(cf)

def read_bdata(folder,Bnum): 
    prefix="BUOY_"
    bname=prefix+Bnum
    print("Reading data for "+bname)
    file=folder+bname+'.csv'
    D=pd.read_csv(file)
    Xib=np.array(D['Lon'])
    Yib=np.array(D['Lat'])
    Tib=np.array(D['Date(GMT)'])
    D.head()
    #velocity computation using the buoy data
    dt=15*60 #sampling time step for each buoy observation.
    #Uib and Vib same length as Xib and Yib
    Uib=np.zeros(len(Xib))
    Vib=np.zeros(len(Xib))
    for j in range(len(Xib)-1):
        dlon=Xib[j+1]-Xib[j]
        dlat=Yib[j+1]-Yib[j]
        lat=Yib[j];lon=Xib[j]
        # (dlatm,dlonm)=gf.latlon2meters(lat,lon,dlat,dlon)
        (dlatm,dlonm)=cf.latlon2meters(lat,dlat,dlon)
        Uib[j+1] = dlonm/dt
        Vib[j+1] = dlatm/dt
    if Bnum=='03':
        #making the time index in datetime with correct formatiing
        Tib_rest=pd.to_datetime(Tib[:],format="%d-%m-%Y %H:%M:%S", errors='coerce')
        #index NaT values
        NaT_index=Tib_rest.get_indexer_for(['NaT']) #since dates like 01-05-2014 are parsed as NaT
        Tib_rest=Tib_rest.to_numpy()
        Tib_main=pd.to_datetime(Tib[:],format="%d-%m-%Y", errors='coerce')
        Tib_main=Tib_main[NaT_index]
        Tib_main=Tib_main.to_numpy()
        #  now insert Tib_main in Tib_rest at the NaT_index
        Tib_rest[NaT_index]=Tib_main
        Tib=pd.to_datetime(Tib_rest,format="%Y-%m-%d %H:%M:%S")
    else:
        Tib=pd.to_datetime(Tib[:],format="%d-%m-%Y %H:%M")
    return(Xib,Yib,Uib,Vib,Tib)

def get_time_index():
    #we are doing for buoys 03 and 16 and for May 2014 since then the buoys are in Spitsbergen region
    #time indexing for the buoy data
    osdate=datetime.datetime(2014, 3, 15)
    sdate_index=int(96*47) #47 days since 15th March 2014 is 1st May 2014
    dt=datetime.timedelta(minutes=int(sdate_index*15))
    sdate=osdate+dt
    #15 days of data
    edate_index=int(sdate_index+96*15)
    return(sdate_index,edate_index)
    
def get_buoy_tidal_data(folder,Bnum):
    dt=15 #mins
    sampling_period=dt/60 #hours    
    (Xib,Yib,Uib,Vib,Tib)=read_bdata(folder,Bnum)
    (sdate_index,edate_index)=get_time_index()
    Xib=Xib[sdate_index:edate_index]
    Yib=Yib[sdate_index:edate_index]
    Uib=Uib[sdate_index:edate_index]
    Vib=Vib[sdate_index:edate_index]
    Tib=Tib[sdate_index:edate_index]
    #bandpass filtering the data to obtain the tidal signal
    (Uibt,Uib,Uibo)=cf.BPfilter (sampling_period,Uib)
    (Vibt,Vib,Vibo)=cf.BPfilter (sampling_period,Vib)
    return(Xib,Yib,Uibt,Vibt,Tib)

def get_median_buoy(Xib,Yib):
    Xib=Xib.sort()
    Yib=Yib.sort()
    Xbm=np.median(Xib)
    Ybm=np.median(Yib)
    return(Xbm,Ybm)
    
#%%
if __name__ == "__main__":
    #reading the buoy data from lars
    #change based on location of the data
    folder='/u/vasulkar/p_emodnet_amey/Tidal_ellipse_calibration/Buoy_data_excel/procesd_buoy_loc_data_Lars/'
    Bnum='16'
    dt=15 #mins
    sampling_period=dt/60 #hours
    (Xib,Yib,Uibt,Vibt,Tib)=get_buoy_tidal_data(folder,Bnum)

    #test plot Uibt ever  hour
    plt.figure()
    plt.plot(Uibt[:])

    # %%
    #now doing a fourier transform on the data to obtain the amplitude and phase of
    #M2 S2 K1 O1
    (Ubt_amp,Ubt_ph,U_fvec,U_tvec)=cf.FFTsig(Uibt,sampling_period)
    (Vbt_amp,Vbt_ph,V_fvec,V_tvec)=cf.FFTsig(Vibt,sampling_period)

    #plotting the FFT of the data
    #with vline for tidal frequencies
    TC=cf.Tidal_const()
    #finding the index of the tidal frequencies in the time vector
    TCval=TC.values()
    TClabel=TC.keys()
    index=[]
    # for i in range(len(TCval)):
    #     index.append(np.argmin(np.abs(U_fvec-TCval[i])))


    #%%

    #plotting vlines for all TC
    TCval=TC.values()
    TClabel=TC.keys()
    plt.figure()
    plt.plot(U_tvec,Ubt_amp)
    # plt.text(x=TCval[0],y=0,s=U_tvec[index[0]])
    plt.vlines(x=TCval,ymin=0,ymax=np.max(Ubt_amp),color='r')
    # plt.text(x=TCval,y=0,s=TClabel)
    plt.xlim([10,30])
    # %%
