# buoy data preprocessing scripss
import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cpf
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
import shapely.geometry as sgeom
from copy import copy
import matplotlib.transforms as transforms
import datetime
import scipy.ndimage as ndimage
import scipy.signal as signal
from scipy.signal import butter, filtfilt

from pyproj import Proj

def Tidal_const():
    # TC=['M2','S2','O1','K1']
    TC={'M2':12.42,'S2':12.00,'O1':25.82,'K1':23.93}
    return(TC)

# Define a pyproj function to reproject data
def coordinate_conv(x, y, inverse = True):
    # p = Proj('+proj=stere +lat_0=90 +lat_ts=70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs')
    p = Proj('+proj=stere +lat_0=90 +lat_ts=70 +lon_0=-45 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs +type=crs')
    return p(x, y, inverse = inverse)

# A conversion function to convert the latitudes and longitudes to meteres.
# note that this is a simple conversion formula and not very accurate older version.
deg2rad= np.pi/180.0
def latlon2meters(lat,dlat,dlon):
    #lat and lon are changes in lat and lon in deg
    dlatm=dlat*111.32e03
    dlonm=dlon*40075e03*np.cos(deg2rad*lat)/360
    return(dlatm,dlonm)

def m2latlon(lat,dlonm,dlatm):
    dlat=dlatm/(111.32e03)
    dlon=dlonm/(111.32e03*np.cos(deg2rad*lat))
    return(dlon,dlat)

#Adopted from cartopy github
# carotpy functions to plot the grid lines
def find_side(ls, side):
    """
    Given a shapely LineString which is assumed to be rectangular, return the
    line corresponding to a given side of the rectangle.
    
    """
    minx, miny, maxx, maxy = ls.bounds
    points = {'left': [(minx, miny), (minx, maxy)],
              'right': [(maxx, miny), (maxx, maxy)],
              'bottom': [(minx, miny), (maxx, miny)],
              'top': [(minx, maxy), (maxx, maxy)],}
    return sgeom.LineString(points[side])


def lambert_xticks(ax, ticks):
    """Draw ticks on the bottom x-axis of a Lambert Conformal projection."""
    te = lambda xy: xy[0]
    lc = lambda t, n, b: np.vstack((np.zeros(n) + t, np.linspace(b[2], b[3], n))).T
    xticks, xticklabels = _lambert_ticks(ax, ticks, 'bottom', lc, te)
    ax.xaxis.tick_bottom()
    ax.set_xticks(xticks)
    ax.set_xticklabels([ax.xaxis.get_major_formatter()(xtick) for xtick in xticklabels])
    
def lambert_yticks(ax, ticks):
    """Draw ricks on the left y-axis of a Lamber Conformal projection."""
    te = lambda xy: xy[1]
    lc = lambda t, n, b: np.vstack((np.linspace(b[0], b[1], n), np.zeros(n) + t)).T
    yticks, yticklabels = _lambert_ticks(ax, ticks, 'left', lc, te)
    ax.yaxis.tick_left()
    ax.set_yticks(yticks)
    ax.set_yticklabels([ax.yaxis.get_major_formatter()(ytick) for ytick in yticklabels])

def _lambert_ticks(ax, ticks, tick_location, line_constructor, tick_extractor):
    """Get the tick locations and labels for an axis of a Lambert Conformal projection."""
    outline_patch = sgeom.LineString(ax.outline_patch.get_path().vertices.tolist())
    axis = find_side(outline_patch, tick_location)
    n_steps = 30
    extent = ax.get_extent(ccrs.PlateCarree())
    _ticks = []
    for t in ticks:
        xy = line_constructor(t, n_steps, extent)
        proj_xyz = ax.projection.transform_points(ccrs.Geodetic(), xy[:, 0], xy[:, 1])
        xyt = proj_xyz[..., :2]
        ls = sgeom.LineString(xyt.tolist())
        locs = axis.intersection(ls)
        if not locs:
            tick = [None]
        else:
            tick = tick_extractor(locs.xy)
        _ticks.append(tick[0])
    # Remove ticks that aren't visible:    
    ticklabels = copy(ticks)
    while True:
        try:
            index = _ticks.index(None)
        except ValueError:
            break
        _ticks.pop(index)
        ticklabels.pop(index)
    return _ticks, ticklabels



def BPfilter (sampling_period,px):
    '''
    sampling period in hours	
    '''
    # filter frequencies from 10h to 30h
    ts_sampling=sampling_period*3600 # 3 hours
    f1=1/(30*3600) # cut off lower
    f2=1/(10*3600) # cut off higher
    nyquist=1/(2*ts_sampling)    
    normal_cutoff = [f1 / nyquist,f2 / nyquist]
    b,a=butter(4,normal_cutoff,btype='band',analog=False)
    y=filtfilt(b,a,px)
    return (y,px,px-y)

# Below is the old function with convolution 
# deprecated: use HPfilter instead
# def LPfilter (numtaps,px):     
#   window=signal.hamming(numtaps)
#   window=window/sum(window)
#   pxfilter=signal.convolve(px, window, mode='valid') 
#   edge=int(numtaps/2)
#   pxres=px[edge:-edge]-pxfilter
#   px=px[edge:-edge] #shortening px for later use.
#   return (pxres,px,pxfilter)

#subroutine for FFT 
def FFTsig(y,ts):
    N=len(y)
    Yamp=abs(np.fft.fft(y))/N
    Ypha=np.angle(np.fft.fft(y),deg=True)
    if (N%2==1):
        N1=int((N-1)/2)
        Yampplt=Yamp[:N1]
        Yphaplt=Ypha[:N1]
    else:
        N1=int((N/2))
        Yampplt=Yamp[:N1]
        Yphaplt=Ypha[:N1]
    #sampling frequency
    ts=ts*3600 #convert to seconds
    fs=1/ts #ts is sampling time in seconds
    f_nq=fs/2 #Nyquist frequency (Fmax)
    fvec=np.linspace(0,f_nq,N1)
    tvec=1/fvec[1:]
    tvec=np.append(np.inf,tvec)
    tvec=tvec/3600 #convert to hours
    return(Yampplt,Yphaplt,fvec,tvec)

##getting mid location in flkoart
def get_midloc_in_float(mid_loc):
    mid_loc_vec=np.zeros((2,1))
    for elem in mid_loc:
        if isinstance(elem,str)==True:
            a=eval(elem)
            a=np.asarray(a)
            mid_loc_vec=np.append(mid_loc_vec,a.reshape((2,1)),axis=1)
        elif np.isnan(elem)==True:
            a=np.array([np.nan,np.nan])
            mid_loc_vec=np.append(mid_loc_vec,a.reshape((2,1)),axis=1)
    mid_loc_vec=mid_loc_vec[:,1:]
    mid_loc_vec=mid_loc_vec.T
    return(mid_loc_vec)

